@@ -1,15 +1,32 @@
 """
 Holds the alternate method used in VarianceEstimator
 """
+# Standard library imports
+import warnings
+
+# Third party imports
 from pyomo.core import log, value
 from pyomo.environ import Objective, Param, SolverFactory
 
-from kipet.core_methods.ResultsObject import ResultsObject
+# KIPET library imports
+from kipet.estimator_tools.results_object import ResultsObject
 
 
 def run_alternate_method(var_est_object, solver, run_opt_kwargs):
-    """Calls the alternative method - Short et al 2020"""
-    
+    """Calls the alternative method - Short et al 2020
+
+    This is an improved method for determining the component variances. This method has been removed from the
+    VarianceEstimator class for simplification.
+
+    :param VarianceEstimator var_est_object: The variance estimation object
+    :param str solver: The solver being used (currently not used)
+    :param dict run_opt_kwargs: The dict of user settings passed on from the ReactionModel
+
+    :return results: The results from the variance estimation
+    :rtype: ResultsObject
+
+    """
+    # Unpack the keyword arguments
     solver_opts = run_opt_kwargs.pop('solver_opts', dict())
     init_sigmas = run_opt_kwargs.pop('initial_sigmas', float())
     tee = run_opt_kwargs.pop('tee', False)
@@ -17,26 +34,24 @@ def run_alternate_method(var_est_object, solver, run_opt_kwargs):
     A = run_opt_kwargs.pop('subset_lambdas', None)
     secant_point2 = run_opt_kwargs.pop('secant_point', None)
     individual_species = run_opt_kwargs.pop('individual_species', False)
-    
+
+    # Solver is fixed to ipopt
     solver = 'ipopt'
-    
+
     nu_squared = var_est_object.solve_max_device_variance(
                                            solver,
                                            tee=tee,
                                            subset_lambdas=A, 
                                            solver_opts=solver_opts)
-        
+
+    second_point = init_sigmas*10
     if secant_point2 is not None:
         second_point = secant_point2
-    else:
-        second_point = init_sigmas*10
+
     itersigma = dict()
-    
-    itersigma[1] = init_sigmas
     itersigma[0] = second_point
-    
+    itersigma[1] = init_sigmas
     iterdelta = dict()
-    
     count = 1
     tol = tol
     funcval = 1000
@@ -50,7 +65,7 @@ def run_alternate_method(var_est_object, solver, run_opt_kwargs):
                                             init_sigmas=itersigma[0])
 
     while abs(funcval) >= tol:
-        print("Overall sigma value at iteration", count,": ", itersigma[count])
+        print("Overall sigma value at iteration", count, ": ", itersigma[count])
         
         new_delta = _solve_delta_given_sigma(var_est_object,
                                              solver, 
@@ -58,15 +73,15 @@ def run_alternate_method(var_est_object, solver, run_opt_kwargs):
                                              subset_lambdas=A, 
                                              solver_opts=solver_opts, 
                                              init_sigmas=itersigma[count])
+
         print("New delta_sq val: ", new_delta)
-        
         iterdelta[count] = new_delta
         
         def func1(nu_squared, new_delta, init_sigmas):
             sigmult = 0
-            nwp = len(var_est_object._meas_lambdas)
-            for l in var_est_object._meas_lambdas:
-                for k in var_est_object.component_set:
+            nwp = len(var_est_object.model.meas_lambdas)
+            for l in var_est_object.model.meas_lambdas:
+                for k in var_est_object.comps['unknown_absorbance']:
                    sigmult += value(var_est_object.model.S[l, k])
             funcval = nu_squared - new_delta - init_sigmas*(sigmult/nwp)
             return funcval, sigmult
@@ -94,38 +109,42 @@ def run_alternate_method(var_est_object, solver, run_opt_kwargs):
                                           delta=new_delta)
     else:
         print("The overall model variance is: ", itersigma[count])
-        #sigma_vals = {k: abs(itersigma[count]) for k in component_set}
         
         sigma_vals = {}
-        for k in var_est_object.component_set:
+        for k in var_est_object.comps['unknown_absorbance']:
             sigma_vals[k] = abs(itersigma[count])
         
     print(f'sigma_vals: {sigma_vals}')
     results = ResultsObject()
-    
-    vars_to_load = ['Z', 'dZdt', 'X', 'dXdt', 'C', 'Cs', 'S', 'Y']
-    if not hasattr(var_est_object, '_abs_components'):
-        vars_to_load.remove('Cs')
-    results.load_from_pyomo_model(var_est_object.model, to_load=vars_to_load)
-     
-    results.P = {name: var_est_object.model.P[name].value for name in var_est_object.model.parameter_names}
+    results.load_from_pyomo_model(var_est_object.model)
     results.sigma_sq = sigma_vals
     results.sigma_sq['device'] = new_delta
 
     return results
 
-def run_direct_sigmas_method(var_est_object, solver, run_opt_kwargs):
-    """Calls the direct sigma method"""
-        
+
+def run_direct_sigmas_method(var_est_object, solver, run_opt_kwargs, fixed=False):
+    """"Calls the direct sigmas method
+
+    :param VarianceEstimator var_est_object: The variance estimation object
+    :param str solver: The solver being used
+    :param dict run_opt_kwargs: The dict of user settings passed on from the ReactionModel
+
+    :return results: The results from the variance estimation
+    :rtype: ResultsObject
+
+    """
     solver_opts = run_opt_kwargs.pop('solver_opts', dict())
     tee = run_opt_kwargs.pop('tee', False)
-    A = run_opt_kwargs.pop('subset_lambdas', None)
+    A = run_opt_kwargs.pop('freq_subset_lambdas', None)
     fixed_device_var = run_opt_kwargs.pop('fixed_device_variance', None)
     device_range = run_opt_kwargs.pop('device_range', None)
     num_points = run_opt_kwargs.pop('num_points', None)
     
     print("Solving for sigmas assuming known device variances")
-    direct_or_it = "it"
+    
+    print('Device range')
+    print(device_range)
     
     if device_range:
         if not isinstance(device_range, tuple):
@@ -137,6 +156,12 @@ def run_direct_sigmas_method(var_est_object, solver, run_opt_kwargs):
             raise Exception
         else:
             print("Device range means that we will solve iteratively for different delta values in that range")
+    
+    else:
+        fixed = True
+        if fixed_device_var is None:
+            raise ValueError("If using fixed variance, this needs to be provided.")
+    
     if device_range and not num_points:
         print("Need to specify the number of points that we wish to evaluate in the device range")
     if not num_points:
@@ -144,10 +169,9 @@ def run_direct_sigmas_method(var_est_object, solver, run_opt_kwargs):
     elif not isinstance(num_points, int):  
         print("num_points needs to be an integer!")
         raise Exception
-    if not device_range:
-        device_range = list() 
+        
     if not device_range and not num_points:
-        direct_or_it = "direct"
+        
         print("assessing for the value of delta provided")
         if not fixed_device_var:
             print("If iterative method not selected then need to provide fixed device variance (delta**2)")
@@ -156,7 +180,10 @@ def run_direct_sigmas_method(var_est_object, solver, run_opt_kwargs):
             if not isinstance(fixed_device_var, float):
                 raise Exception("fixed device variance needs to be of type float")
     
-    if direct_or_it in ["it"]:
+    if not fixed:
+        
+        results_sigmas_dict = {}
+        
         dist = abs((device_range[1] - device_range[0])/num_points)
         
         max_likelihood_vals = []
@@ -170,13 +197,17 @@ def run_direct_sigmas_method(var_est_object, solver, run_opt_kwargs):
        
         while delta < device_range[1]:
             
+            results_sigmas_dict[count] = {}
+            
             print(f"Iteration: {count}\tdelta_sq: {delta}")
             max_likelihood_val, sigma_vals, stop_it, results = \
-                _solve_sigma_given_delta(var_est_object,solver, 
-                                              subset_lambdas=A, 
-                                              solver_opts=solver_opts, 
-                                              tee=tee,
-                                              delta=delta)
+                _solve_sigma_given_delta(var_est_object,
+                                         solver, 
+                                         subset_lambdas=A, 
+                                         solver_opts=solver_opts, 
+                                         tee=tee,
+                                         delta=delta
+                                    )
             
             if max_likelihood_val >= 5000:
                 max_likelihood_vals.append(5000)
@@ -187,55 +218,48 @@ def run_direct_sigmas_method(var_est_object, solver, run_opt_kwargs):
             else:
                 iteration_counter.append(count)
                 
+            results_sigmas_dict[count]['delta'] = delta
+            results_sigmas_dict[count]['simgas'] = sigma_vals
+            #results_sigmas_dict[count]['results'] = results
+                
             delta = delta + dist
             count += 1 
+            
+        return results_sigmas_dict
 
     else:
-        max_likelihood_val, sigma_vals, stop_it = \
-            _solve_sigma_given_delta(var_est_object,
-                                          solver, 
-                                          subset_lambdas=A, 
-                                          solver_opts=solver_opts, 
-                                          tee=tee,
-                                          delta=fixed_device_var)
-   
-    results = ResultsObject()
-    
-    vars_to_load = ['Z', 'dZdt', 'X', 'dXdt', 'C', 'Cs', 'S', 'Y']
-    if not hasattr(var_est_object, '_abs_components'):
-        vars_to_load.remove('Cs')
-    results.load_from_pyomo_model(var_est_object.model, to_load=vars_to_load)
-    
-    results.P = {name: var_est_object.model.P[name].value for name in var_est_object.model.parameter_names}
-    results.sigma_sq = sigma_vals
-    results.sigma_sq['device'] = delta
+        # The optimization will be conducted at the fixed value for delta
         
-    return results
+        max_likelihood_val, sigma_vals, stop_it, results = \
+            _solve_sigma_given_delta(var_est_object,
+                                     solver, 
+                                     subset_lambdas=A, 
+                                     solver_opts=solver_opts, 
+                                     tee=tee,
+                                     delta=fixed_device_var
+                                )
+            
+        delta = fixed_device_var
+   
+        results = ResultsObject()
+        results.load_from_pyomo_model(var_est_object.model)
+        results.sigma_sq = sigma_vals
+        results.sigma_sq['device'] = delta
+            
+        return results
+
 
 def _solve_delta_given_sigma(var_est_object, solver, **kwds):
-    """Solves the maximum likelihood formulation with fixed sigmas in order to
-    obtain delta. This formulation is highly unstable as the problem is ill-posed
-    with the optimal solution being that C-Z = 0. Should not use.
+    """Solves the delta with provided variances
 
-       This method is not intended to be used by users directly
+    :param VarianceEstimator var_est_object: The variance estimation object
+    :param str solver: The solver being used (currently not used)
+    :param dict kwds: The dict of user settings passed on from the ReactionModel
 
-    Args:
-        init_sigmas (dict): variances 
-    
-        tee (bool,optional): flag to tell the optimizer whether to stream output
-        to the terminal or not
-    
-        profile_time (bool,optional): flag to tell pyomo to time the construction and solution of the model. 
-        Default False
-    
-        subset_lambdas (array_like,optional): Set of wavelengths to used in initialization problem 
-        (Weifeng paper). Default all wavelengths.
+    :return results: The results from the variance estimation
+    :rtype: ResultsObject
 
-    Returns:
-
-        None
-
-    """   
+    """
     solver_opts = kwds.pop('solver_opts', dict())
     tee = kwds.pop('tee', False)
     set_A = kwds.pop('subset_lambdas', list())
@@ -244,15 +268,15 @@ def _solve_delta_given_sigma(var_est_object, solver, **kwds):
 
     sigmas_sq = dict()
     if not set_A:
-        set_A = var_est_object._meas_lambdas
+        set_A = var_est_object.model.meas_lambdas
     
     if isinstance(sigmas, float):
-        for k in var_est_object.component_set:
+        for k in var_est_object.comps['unknown_absorbance']:
             sigmas_sq[k] = sigmas
     
     elif isinstance(sigmas, dict):
         keys = sigmas.keys()
-        for k in var_est_object.component_set:
+        for k in var_est_object.comps['unknown_absorbance']:
             if k not in keys:
                 sigmas_sq[k] = sigmas
        
@@ -262,21 +286,21 @@ def _solve_delta_given_sigma(var_est_object, solver, **kwds):
     var_est_object._warn_if_D_negative()  
    
     obj = 0.0
-    ntp = len(var_est_object._meas_times)
-    nwp = len(var_est_object._meas_lambdas) 
+    ntp = len(var_est_object.model.times_spectral)
+    nwp = len(var_est_object.model.meas_lambdas) 
     inlog = 0
-    nc = len(var_est_object.component_set)
-    for t in var_est_object._meas_times:
+    nc = len(var_est_object.comps['unknown_absorbance'])
+    for t in var_est_object.model.times_spectral:
         for l in set_A:
-            D_bar = sum(var_est_object.model.C[t, k] * var_est_object.model.S[l, k] for k in var_est_object.component_set)
+            D_bar = sum(var_est_object.model.C[t, k] * var_est_object.model.S[l, k] for k in var_est_object.comps['unknown_absorbance'])
             #D_bar = sum(var_est_object.model.C[t, k] * var_est_object.model.S[l, k] for k in var_est_object._sublist_components)
             inlog += (var_est_object.model.D[t, l] - D_bar)**2
     # Orig had sublist in both parts - is this an error?
 
     # Concentration - correct
     # Move this to objectives module
-    for t in var_est_object._meas_times:
-        for k in var_est_object._sublist_components:
+    for t in var_est_object.model.times_spectral:
+        for k in var_est_object.comps['unknown_absorbance']:
             #obj += conc_objective(var_est_object.model, sigma=sigmas_sq)
             obj += 0.5*((var_est_object.model.C[t, k] - var_est_object.model.Z[t, k])**2)/(sigmas_sq[k])
             
@@ -298,9 +322,9 @@ def _solve_delta_given_sigma(var_est_object, solver, **kwds):
         print(k, v.value)
         
     etaTeta = 0
-    for t in var_est_object._meas_times:
+    for t in var_est_object.model.times_spectral:
         for l in set_A:
-            D_bar = sum(value(var_est_object.model.C[t, k]) * value(var_est_object.model.S[l, k]) for k in var_est_object.component_set)
+            D_bar = sum(value(var_est_object.model.C[t, k]) * value(var_est_object.model.S[l, k]) for k in var_est_object.comps['unknown_absorbance'])
             etaTeta += (value(var_est_object.model.D[t, l]) - D_bar)**2
     
     deltasq = etaTeta/(ntp*nwp)  
@@ -309,83 +333,73 @@ def _solve_delta_given_sigma(var_est_object, solver, **kwds):
     return deltasq
 
 def _solve_sigma_given_delta(var_est_object, solver, **kwds):
-    """Solves the maximum likelihood formulation to determine the model variance from a
-    given device variance.
+    """Solves the delta (device variance) with provided variances
 
-       This method is not intended to be used by users directly
+    :param VarianceEstimator var_est_object: The variance estimation object
+    :param str solver: The solver being used (currently not used)
+    :param dict kwds: The dict of user settings passed on from the ReactionModel
 
-    Args:
-        delta (float): the device variance squared 
-    
-        tee (bool,optional): flag to tell the optimizer whether to stream output
-        to the terminal or not
-    
-        profile_time (bool,optional): flag to tell pyomo to time the construction and solution of the model. 
-        Default False
-    
-        subset_lambdas (array_like,optional): Set of wavelengths to used in initialization problem 
-        (Weifeng paper). Default all wavelengths.
-        
-        solver_opts (dict, optional): dictionary containing solver options for IPOPT
+    :return residuals: The results from the variance estimation
+    :return variances_dict: dictionary containing the model variance values
+    :return stop_it: boolean indicator showing whether no solution was found (True) or if there is a solution (False)
+    :return solver_results: dictionary containing solver options for IPOPT
 
-    Returns:
+    :rtype: ResultsObject
 
-        residuals (float): the objective function value from the optimization
-        
-        variancesdict (dict): dictionary containing the model variance values
-        
-        stop_it (bool): boolean indicator showing whether no solution was found 
-                            (True) or if there is a solution (False)
-                            
-        solver_results: Variance estimation model results, similar to VarianceEstimator 
-                                    results from previous method
+    :Keyword Args:
+
+        - delta (float): the device variance
+        - tee (bool,optional): flag to tell the optimizer whether to stream output to the terminal or not
+        - profile_time (bool,optional): flag to tell pyomo to time the construction and solution of the model. Default False
+        - subset_lambdas (array_like,optional): Set of wavelengths to used in initialization problem (Weifeng paper). Default all wavelengths.
+        - solver_opts (dict, optional): dictionary containing solver options for IPOPT
 
     """   
     solver_opts = kwds.pop('solver_opts', dict())
     tee = kwds.pop('tee', False)
     set_A = kwds.pop('subset_lambdas', list())
     profile_time = kwds.pop('profile_time', False)
-    delta = kwds.pop('delta', dict())
-    species_list = kwds.pop('subset_components', None)
+    delta_sq = kwds.pop('delta', dict())
+    #species_list = kwds.pop('subset_components', None)
 
     model = var_est_object.model.clone()
 
     if not set_A:
-        set_A = var_est_object._meas_lambdas
+        set_A = var_est_object.model.meas_lambdas
         
-    if not var_est_object._sublist_components:
-        list_components = []
-        if species_list is None:
-            list_components = [k for k in var_est_object._mixture_components]
+    # if not hasattr(var_est_object, '_sublist_components'):
+    #     list_components = []
+    #     if species_list is None:
+    #         list_components = [k for k in var_est_object._mixture_components]
             
-        else:
-            for k in species_list:
-                if k in var_est_object._mixture_components:
-                    list_components.append(k)
-                else:
-                    warnings.warn("Ignored {} since is not a mixture component of the model".format(k))
+    #     else:
+    #         for k in species_list:
+    #             if k in var_est_object._mixture_components:
+    #                 list_components.append(k)
+    #             else:
+    #                 warnings.warn("Ignored {} since is not a mixture component of the model".format(k))
 
-        var_est_object._sublist_components = list_components
+    #     var_est_object._sublist_components = list_components
     
     var_est_object._warn_if_D_negative()  
-    ntp = len(var_est_object._meas_times)
+    ntp = len(var_est_object.model.times_spectral)
     obj = 0.0
    
-    for t in var_est_object._meas_times:
+    for t in var_est_object.model.times_spectral:
         for l in set_A:
-            D_bar = sum(var_est_object.model.C[t, k] * var_est_object.model.S[l, k] for k in var_est_object.component_set)
-            obj += 0.5/delta*(var_est_object.model.D[t, l] - D_bar)**2
+            D_bar = sum(var_est_object.model.C[t, k] * var_est_object.model.S[l, k] for k in var_est_object.comps['unknown_absorbance'])
+            obj += 0.5/delta_sq*(var_est_object.model.D[t, l] - D_bar)**2
 
-    inlog = {k: 0 for k in var_est_object.component_set}
+    inlog = {k: 0 for k in var_est_object.comps['unknown_absorbance']}
     var_est_object.model.eps = Param(initialize = 1e-8)  
     
-    variancesdict = {k: 0 for k in var_est_object.component_set}
+    variances_dict = {k: 0 for k in var_est_object.comps['unknown_absorbance']}
             
-    for t in var_est_object._meas_times:
-        for k in var_est_object.component_set:
+    for t in var_est_object.model.times_spectral:
+        for k in var_est_object.comps['unknown_absorbance']:
             inlog[k] += ((var_est_object.model.C[t, k] - var_est_object.model.Z[t, k])**2)
     
-    for k in var_est_object.component_set:
+    for k in var_est_object.comps['unknown_absorbance']:
         obj += 0.5*ntp*log(inlog[k]/ntp + var_est_object.model.eps)
     
     var_est_object.model.init_objective = Objective(expr=obj)
@@ -400,13 +414,13 @@ def _solve_sigma_given_delta(var_est_object, solver, **kwds):
                                report_timing=profile_time)
 
         residuals = (value(var_est_object.model.init_objective))
-        for t in var_est_object._allmeas_times:
-            for k in var_est_object.component_set:
-                variancesdict[k] += 1 / ntp*((value(var_est_object.model.C[t, k]) - value(var_est_object.model.Z[t, k]))**2)
+        for t in var_est_object.model.times_spectral:
+            for k in var_est_object.comps['unknown_absorbance']:
+                variances_dict[k] += 1 / ntp*((value(var_est_object.model.C[t, k]) - value(var_est_object.model.Z[t, k]))**2)
         
         print("Variances")
-        for k in var_est_object.component_set:
-            print(k, variancesdict[k])
+        for k in var_est_object.comps['unknown_absorbance']:
+            print(k, variances_dict[k])
         
         print("Parameter estimates")
         for k, v in var_est_object.model.P.items():
@@ -416,7 +430,7 @@ def _solve_sigma_given_delta(var_est_object, solver, **kwds):
         
     except:
         print("FAILED AT THIS ITERATION")
-        variancesdict = None
+        variances_dict = None
         residuals = 0
         stop_it = True
         solver_results = None
@@ -428,4 +442,4 @@ def _solve_sigma_given_delta(var_est_object, solver, **kwds):
     var_est_object.model.del_component('init_objective')
     var_est_object.model.del_component('eps')
 
-    return residuals, variancesdict, stop_it, solver_results
\ No newline at end of file
+    return residuals, variances_dict, stop_it, solver_results
